@@ -21,7 +21,6 @@
 import gtk
 
 import copy
-import traceback
 import logging
 
 from virtManager import util
@@ -35,9 +34,6 @@ PAGE_NAME   = 0
 PAGE_FORMAT = 1
 
 class vmmCreatePool(vmmGObjectUI):
-    __gsignals__ = {
-    }
-
     def __init__(self, conn):
         vmmGObjectUI.__init__(self,
                               "vmm-create-pool.glade",
@@ -59,6 +55,7 @@ class vmmCreatePool(vmmGObjectUI):
 
             "on_pool_name_activate": self.forward,
             "on_pool_hostname_activate" : self.hostname_changed,
+            "on_pool_iqn_chk_toggled": self.iqn_toggled,
 
             "on_pool_name_focus_in_event": (self.update_doc, "name",
                                             "pool-info1"),
@@ -92,37 +89,47 @@ class vmmCreatePool(vmmGObjectUI):
 
             "on_pool_hostname_focus_in_event": (self.update_doc, "host",
                                                 "pool-info2"),
-            "on_pool_build_focus_in_event": (self.update_build_doc)
+            "on_pool_build_focus_in_event": (self.update_build_doc),
+
+            "on_pool_iqn_focus_in_event": (self.update_doc, "iqn",
+                                           "pool-info2"),
         })
-        util.bind_escape_key_close(self)
+        self.bind_escape_key_close()
 
         # XXX: Help docs useless/out of date
-        self.window.get_widget("pool-help").hide()
+        self.widget("pool-help").hide()
         finish_img = gtk.image_new_from_stock(gtk.STOCK_QUIT,
                                               gtk.ICON_SIZE_BUTTON)
-        self.window.get_widget("pool-finish").set_image(finish_img)
+        self.widget("pool-finish").set_image(finish_img)
 
         self.set_initial_state()
 
-    def show(self):
+    def show(self, parent):
         self.reset_state()
+        self.topwin.set_transient_for(parent)
         self.topwin.present()
 
     def close(self, ignore1=None, ignore2=None):
         self.topwin.hide()
         return 1
 
-    def set_initial_state(self):
-        self.window.get_widget("pool-pages").set_show_tabs(False)
+    def _cleanup(self):
+        self.close()
 
-        type_list = self.window.get_widget("pool-type")
+        self.conn = None
+        self._pool = None
+
+    def set_initial_state(self):
+        self.widget("pool-pages").set_show_tabs(False)
+
+        type_list = self.widget("pool-type")
         type_model = gtk.ListStore(str, str)
         type_list.set_model(type_model)
         text1 = gtk.CellRendererText()
         type_list.pack_start(text1, True)
         type_list.add_attribute(text1, 'text', 1)
 
-        format_list = self.window.get_widget("pool-format")
+        format_list = self.widget("pool-format")
         format_model = gtk.ListStore(str, str)
         format_list.set_model(format_model)
         text2 = gtk.CellRendererText()
@@ -130,7 +137,7 @@ class vmmCreatePool(vmmGObjectUI):
         format_list.add_attribute(text2, 'text', 1)
 
         # Target path combo box entry
-        target_list = self.window.get_widget("pool-target-path")
+        target_list = self.widget("pool-target-path")
         # target_path, Label, pool class instance
         target_model = gtk.ListStore(str, str, object)
         target_model.set_sort_column_id(0, gtk.SORT_ASCENDING)
@@ -140,7 +147,7 @@ class vmmCreatePool(vmmGObjectUI):
                                   "target_path", "pool-info2")
 
         # Source path combo box entry
-        source_list = self.window.get_widget("pool-source-path")
+        source_list = self.widget("pool-source-path")
         # source_path, Label, pool class instance
         source_model = gtk.ListStore(str, str, object)
         source_model.set_sort_column_id(0, gtk.SORT_ASCENDING)
@@ -151,50 +158,59 @@ class vmmCreatePool(vmmGObjectUI):
 
         self.populate_pool_type()
 
-        self.window.get_widget("pool-info-box1").modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("grey"))
-        self.window.get_widget("pool-info-box2").modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("grey"))
+        self.widget("pool-info-box1").modify_bg(gtk.STATE_NORMAL,
+                                                gtk.gdk.color_parse("grey"))
+        self.widget("pool-info-box2").modify_bg(gtk.STATE_NORMAL,
+                                                gtk.gdk.color_parse("grey"))
 
     def reset_state(self):
-        self.window.get_widget("pool-pages").set_current_page(0)
-        self.window.get_widget("pool-forward").show()
-        self.window.get_widget("pool-finish").hide()
-        self.window.get_widget("pool-back").set_sensitive(False)
+        self.widget("pool-pages").set_current_page(0)
+        self.widget("pool-forward").show()
+        self.widget("pool-finish").hide()
+        self.widget("pool-back").set_sensitive(False)
 
-        self.window.get_widget("pool-name").set_text("")
-        self.window.get_widget("pool-name").grab_focus()
-        self.window.get_widget("pool-type").set_active(0)
-        self.window.get_widget("pool-target-path").child.set_text("")
-        self.window.get_widget("pool-source-path").child.set_text("")
-        self.window.get_widget("pool-hostname").set_text("")
-        self.window.get_widget("pool-format").set_active(-1)
-        self.window.get_widget("pool-build").set_sensitive(True)
-        self.window.get_widget("pool-build").set_active(False)
+        self.widget("pool-name").set_text("")
+        self.widget("pool-name").grab_focus()
+        self.widget("pool-type").set_active(0)
+        self.widget("pool-target-path").child.set_text("")
+        self.widget("pool-source-path").child.set_text("")
+        self.widget("pool-hostname").set_text("")
+        self.widget("pool-iqn-chk").set_active(False)
+        self.widget("pool-iqn-chk").toggled()
+        self.widget("pool-iqn").set_text("")
+        self.widget("pool-format").set_active(-1)
+        self.widget("pool-build").set_sensitive(True)
+        self.widget("pool-build").set_active(False)
 
 
     def hostname_changed(self, ignore):
         # If a hostname was entered, try to lookup valid pool sources.
         self.populate_pool_sources()
 
+    def iqn_toggled(self, src):
+        self.widget("pool-iqn").set_sensitive(src.get_active())
+
     def populate_pool_type(self):
-        model = self.window.get_widget("pool-type").get_model()
+        model = self.widget("pool-type").get_model()
         model.clear()
         types = Storage.StoragePool.get_pool_types()
         types.sort()
         for typ in types:
-            model.append([typ, "%s: %s" % (typ, Storage.StoragePool.get_pool_type_desc(typ))])
+            model.append([typ, "%s: %s" %
+                         (typ, Storage.StoragePool.get_pool_type_desc(typ))])
 
     def populate_pool_format(self, formats):
-        model = self.window.get_widget("pool-format").get_model()
+        model = self.widget("pool-format").get_model()
         model.clear()
         for f in formats:
             model.append([f, f])
 
     def populate_pool_sources(self):
-        source_list = self.window.get_widget("pool-source-path")
+        source_list = self.widget("pool-source-path")
         source_model = source_list.get_model()
         source_model.clear()
 
-        target_list = self.window.get_widget("pool-target-path")
+        target_list = self.widget("pool-target-path")
         target_model = target_list.get_model()
         target_model.clear()
 
@@ -213,6 +229,11 @@ class vmmCreatePool(vmmGObjectUI):
             use_list = target_list
             use_model = target_model
 
+        elif self._pool.type == Storage.StoragePool.TYPE_DISK:
+            entry_list = self.list_disk_devs()
+            use_list = source_list
+            use_model = source_model
+
         elif self._pool.type == Storage.StoragePool.TYPE_NETFS:
             host = self.get_config_host()
             if host:
@@ -229,7 +250,7 @@ class vmmCreatePool(vmmGObjectUI):
             use_list.set_active(0)
 
     def list_scsi_adapters(self):
-        scsi_hosts = self.conn.get_devices("scsi_host")
+        scsi_hosts = self.conn.get_nodedevs("scsi_host")
         host_list = map(lambda dev: dev.host, scsi_hosts)
 
         clean_list = []
@@ -245,6 +266,26 @@ class vmmCreatePool(vmmGObjectUI):
 
         return clean_list
 
+    def list_disk_devs(self):
+        devs = self.conn.get_nodedevs("storage")
+        devlist = []
+        for dev in devs:
+            if dev.drive_type != "disk" or not dev.block:
+                continue
+            devlist.append(dev.block)
+
+        devlist.sort()
+        clean_list = []
+        for dev in devlist:
+            tmppool = copy.copy(self._pool)
+            tmppool.source_path = dev
+
+            entry = [dev, dev, tmppool]
+            if dev not in map(lambda l: l[0], clean_list):
+                clean_list.append(entry)
+
+        return clean_list
+
     def list_pool_sources(self, host=None):
         name = self.get_config_name()
         pool_type = self._pool.type
@@ -260,12 +301,18 @@ class vmmCreatePool(vmmGObjectUI):
         return plist
 
     def show_options_by_pool(self):
+        def show_row(base, do_show):
+            self.widget(base + "-label").set_property("visible", do_show)
+            self.widget(base + "-box").set_property("visible", do_show)
+
         src     = hasattr(self._pool, "source_path")
         src_b   = src and not self.conn.is_remote()
         tgt     = hasattr(self._pool, "target_path")
         tgt_b   = tgt and not self.conn.is_remote()
         host    = hasattr(self._pool, "host")
         fmt     = hasattr(self._pool, "formats")
+        iqn     = hasattr(self._pool, "iqn")
+        builddef, buildsens = self.get_build_default()
 
         # Source path broswing is meaningless for net pools
         if self._pool.type in [Storage.StoragePool.TYPE_NETFS,
@@ -273,31 +320,37 @@ class vmmCreatePool(vmmGObjectUI):
                                Storage.StoragePool.TYPE_SCSI]:
             src_b = False
 
-        self.window.get_widget("pool-target-button").set_sensitive(tgt_b)
-        self.window.get_widget("pool-source-button").set_sensitive(src_b)
-        self.window.get_widget("pool-source-path").set_sensitive(src)
-        self.window.get_widget("pool-hostname").set_sensitive(host)
-        self.window.get_widget("pool-format").set_sensitive(fmt)
-        self.window.get_widget("pool-format").set_active(-1)
+        show_row("pool-target", tgt)
+        show_row("pool-source", src)
+        show_row("pool-hostname", host)
+        show_row("pool-format", fmt)
+        show_row("pool-build", buildsens)
+        show_row("pool-iqn", iqn)
 
+        self.widget("pool-target-path").child.set_text(self._pool.target_path)
+        self.widget("pool-target-button").set_sensitive(tgt_b)
+        self.widget("pool-source-button").set_sensitive(src_b)
+        self.widget("pool-build").set_active(builddef)
+
+        self.widget("pool-format").set_active(-1)
         if fmt:
             self.populate_pool_format(getattr(self._pool, "formats"))
-            self.window.get_widget("pool-format").set_active(0)
+            self.widget("pool-format").set_active(0)
 
         self.populate_pool_sources()
 
 
     def get_config_type(self):
-        typ = self.window.get_widget("pool-type")
+        typ = self.widget("pool-type")
         if typ.get_active_iter() != None:
             return typ.get_model().get_value(typ.get_active_iter(), 0)
         return None
 
     def get_config_name(self):
-        return self.window.get_widget("pool-name").get_text()
+        return self.widget("pool-name").get_text()
 
     def get_config_target_path(self):
-        src = self.window.get_widget("pool-target-path")
+        src = self.widget("pool-target-path")
         if not src.get_property("sensitive"):
             return None
 
@@ -310,7 +363,7 @@ class vmmCreatePool(vmmGObjectUI):
         return src.child.get_text()
 
     def get_config_source_path(self):
-        src = self.window.get_widget("pool-source-path")
+        src = self.widget("pool-source-path")
         if not src.get_property("sensitive"):
             return None
 
@@ -323,19 +376,25 @@ class vmmCreatePool(vmmGObjectUI):
         return src.child.get_text().strip()
 
     def get_config_host(self):
-        host = self.window.get_widget("pool-hostname")
+        host = self.widget("pool-hostname")
         if host.get_property("sensitive"):
             return host.get_text().strip()
         return None
 
     def get_config_format(self):
-        format_combo = self.window.get_widget("pool-format")
+        format_combo = self.widget("pool-format")
         model = format_combo.get_model()
         if format_combo.get_active_iter() != None:
             model = format_combo.get_model()
             return model.get_value(format_combo.get_active_iter(), 0)
         return None
 
+    def get_config_iqn(self):
+        iqn = self.widget("pool-iqn")
+        if iqn.get_sensitive() and iqn.get_property("visible"):
+            return iqn.get_text().strip()
+        return None
+
     def get_build_default(self):
         """ Return (default value, whether build option can be changed)"""
         if not self._pool:
@@ -358,44 +417,44 @@ class vmmCreatePool(vmmGObjectUI):
         source = self._browse_file(_("Choose source path"),
                                    startfolder="/dev", foldermode=False)
         if source:
-            self.window.get_widget("pool-source-path").child.set_text(source)
+            self.widget("pool-source-path").child.set_text(source)
 
     def browse_target_path(self, ignore1=None):
         target = self._browse_file(_("Choose target directory"),
                                    startfolder="/var/lib/libvirt",
                                    foldermode=True)
         if target:
-            self.window.get_widget("pool-target-path").child.set_text(target)
+            self.widget("pool-target-path").child.set_text(target)
 
 
     def forward(self, ignore=None):
-        notebook = self.window.get_widget("pool-pages")
+        notebook = self.widget("pool-pages")
         try:
             if(self.validate(notebook.get_current_page()) != True):
                 return
             if notebook.get_current_page() == PAGE_FORMAT:
                 self.finish()
             else:
-                self.window.get_widget("pool-forward").grab_focus()
+                self.widget("pool-forward").grab_focus()
                 notebook.next_page()
         except Exception, e:
-            self.err.show_err(_("Uncaught error validating input: %s") % str(e),
-                              "".join(traceback.format_exc()))
+            self.err.show_err(_("Uncaught error validating input: %s") % str(e))
             return
 
     def back(self, ignore=None):
-        self.window.get_widget("pool-finish").hide()
-        self.window.get_widget("pool-forward").show()
-        self.window.get_widget("pool-pages").prev_page()
+        self.widget("pool-finish").hide()
+        self.widget("pool-forward").show()
+        self.widget("pool-pages").prev_page()
 
     def finish(self):
         self.topwin.set_sensitive(False)
         self.topwin.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
 
         progWin = vmmAsyncJob(self._async_pool_create, [],
-                              title=_("Creating storage pool..."),
-                              text=_("Creating the storage pool may take a "
-                                     "while..."))
+                              _("Creating storage pool..."),
+                              _("Creating the storage pool may take a "
+                                "while..."),
+                              self.topwin)
         error, details = progWin.run()
 
         self.topwin.set_sensitive(True)
@@ -403,12 +462,12 @@ class vmmCreatePool(vmmGObjectUI):
 
         if error:
             error = _("Error creating pool: %s") % error
-            self.err.show_err(error, error + "\n" + details)
+            self.err.show_err(error,
+                              details=details)
         else:
             self.close()
 
-    def _async_pool_create(self, asyncjob, *args, **kwargs):
-        print args, kwargs
+    def _async_pool_create(self, asyncjob):
         newconn = None
 
         # Open a seperate connection to install on since this is async
@@ -417,26 +476,22 @@ class vmmCreatePool(vmmGObjectUI):
         self._pool.conn = newconn
 
         logging.debug("Starting backround pool creation.")
-        build = self.window.get_widget("pool-build").get_active()
+        build = self.widget("pool-build").get_active()
         poolobj = self._pool.install(create=True, meter=meter, build=build)
         poolobj.setAutostart(True)
-        logging.debug("Pool creating succeeded.")
+        logging.debug("Pool creation succeeded")
 
     def page_changed(self, notebook_ignore, page_ignore, page_number):
         if page_number == PAGE_NAME:
-            self.window.get_widget("pool-back").set_sensitive(False)
-            self.window.get_widget("pool-finish").hide()
-            self.window.get_widget("pool-forward").show()
-            self.window.get_widget("pool-forward").grab_focus()
+            self.widget("pool-back").set_sensitive(False)
+            self.widget("pool-finish").hide()
+            self.widget("pool-forward").show()
+            self.widget("pool-forward").grab_focus()
         elif page_number == PAGE_FORMAT:
-            self.window.get_widget("pool-target-path").child.set_text(self._pool.target_path)
-            self.window.get_widget("pool-back").set_sensitive(True)
-            buildret = self.get_build_default()
-            self.window.get_widget("pool-build").set_sensitive(buildret[1])
-            self.window.get_widget("pool-build").set_active(buildret[0])
-            self.window.get_widget("pool-finish").show()
-            self.window.get_widget("pool-finish").grab_focus()
-            self.window.get_widget("pool-forward").hide()
+            self.widget("pool-back").set_sensitive(True)
+            self.widget("pool-finish").show()
+            self.widget("pool-finish").grab_focus()
+            self.widget("pool-forward").hide()
             self.show_options_by_pool()
 
     def get_pool_to_validate(self):
@@ -445,8 +500,8 @@ class vmmCreatePool(vmmGObjectUI):
         For most pools this will be the one we built after step 1, but for
         pools we find via FindPoolSources, this will be different
         """
-        source_list = self.window.get_widget("pool-source-path")
-        target_list = self.window.get_widget("pool-target-path")
+        source_list = self.widget("pool-source-path")
+        target_list = self.widget("pool-target-path")
 
         pool = copy.copy(self._pool)
 
@@ -472,10 +527,11 @@ class vmmCreatePool(vmmGObjectUI):
             return True
 
         elif page == PAGE_FORMAT:
-            target = self.get_config_target_path()
-            host   = self.get_config_host()
-            source = self.get_config_source_path()
-            fmt    = self.get_config_format()
+            target  = self.get_config_target_path()
+            host    = self.get_config_host()
+            source  = self.get_config_source_path()
+            fmt     = self.get_config_format()
+            iqn     = self.get_config_iqn()
 
             tmppool = self.get_pool_to_validate()
             try:
@@ -486,14 +542,16 @@ class vmmCreatePool(vmmGObjectUI):
                     tmppool.source_path = source
                 if fmt:
                     tmppool.format = fmt
+                if iqn:
+                    tmppool.iqn = iqn
 
                 tmppool.get_xml_config()
             except ValueError, e:
                 return self.err.val_err(_("Pool Parameter Error"), str(e))
 
-            buildval = self.window.get_widget("pool-build").get_active()
-            buildsen = self.window.get_widget("pool-build").get_property(
-                                                                "sensitive")
+            buildval = self.widget("pool-build").get_active()
+            buildsen = (self.widget("pool-build").get_property("sensitive") and
+                        self.widget("pool-build-box").get_property("visible"))
             if buildsen and buildval:
                 ret = self.err.yes_no(_("Building a pool of this type will "
                                         "format the source device. Are you "
@@ -506,7 +564,7 @@ class vmmCreatePool(vmmGObjectUI):
 
     def update_doc(self, ignore1, ignore2, param, infobox):
         doc = self._build_doc_str(param)
-        self.window.get_widget(infobox).set_markup(doc)
+        self.widget(infobox).set_markup(doc)
 
     def update_build_doc(self, ignore1, ignore2):
         doc = ""
@@ -518,7 +576,7 @@ class vmmCreatePool(vmmGObjectUI):
 
         if docstr:
             doc = self._build_doc_str("build", docstr)
-        self.window.get_widget("pool-info2").set_markup(doc)
+        self.widget("pool-info2").set_markup(doc)
 
     def update_doc_changed(self, ignore1, param, infobox):
         # Wrapper for update_doc and 'changed' signal
