@@ -18,13 +18,13 @@
 # MA 02110-1301 USA.
 #
 
-import gobject
-import gtk
-
 import traceback
 import logging
 import threading
 
+import gobject
+import gtk
+
 import virtinst
 import libvirt
 
@@ -47,13 +47,10 @@ def uri_join(uri_tuple):
 
 
 class vmmMigrateDialog(vmmGObjectUI):
-    __gsignals__ = {
-    }
-
     def __init__(self, vm, engine):
         vmmGObjectUI.__init__(self, "vmm-migrate.glade", "vmm-migrate")
         self.vm = vm
-        self.conn = vm.connection
+        self.conn = vm.conn
         self.engine = engine
 
         self.destconn_rows = []
@@ -70,31 +67,46 @@ class vmmMigrateDialog(vmmGObjectUI):
             "on_migrate_set_port_toggled" : self.toggle_set_port,
             "on_migrate_set_maxdowntime_toggled" : self.toggle_set_maxdowntime,
         })
-        util.bind_escape_key_close(self)
+        self.bind_escape_key_close()
 
         blue = gtk.gdk.color_parse("#0072A8")
-        self.window.get_widget("migrate-header").modify_bg(gtk.STATE_NORMAL,
+        self.widget("migrate-header").modify_bg(gtk.STATE_NORMAL,
                                                            blue)
         image = gtk.image_new_from_icon_name("vm_clone_wizard",
                                              gtk.ICON_SIZE_DIALOG)
         image.show()
-        self.window.get_widget("migrate-vm-icon-box").pack_end(image, False)
+        self.widget("migrate-vm-icon-box").pack_end(image, False)
 
         self.init_state()
 
-    def show(self):
+    def show(self, parent):
         self.reset_state()
-        self.topwin.show()
+        self.topwin.resize(1, 1)
+        self.topwin.set_transient_for(parent)
         self.topwin.present()
 
     def close(self, ignore1=None, ignore2=None):
         self.topwin.hide()
+        # If we only do this at show time, operation takes too long and
+        # user actually sees the expander close.
+        self.widget("migrate-advanced-expander").set_expanded(False)
         return 1
 
+    def _cleanup(self):
+        self.close()
+
+        self.vm = None
+        self.conn = None
+        self.engine = None
+        self.destconn_rows = None
+
+        # Not sure why we need to do this manually, but it matters
+        self.widget("migrate-dest").get_model().clear()
+
     def init_state(self):
         # [hostname, conn, can_migrate, tooltip]
         dest_model = gtk.ListStore(str, object, bool, str)
-        dest_combo = self.window.get_widget("migrate-dest")
+        dest_combo = self.widget("migrate-dest")
         dest_combo.set_model(dest_model)
         text = gtk.CellRendererText()
         dest_combo.pack_start(text, True)
@@ -104,38 +116,37 @@ class vmmMigrateDialog(vmmGObjectUI):
         # XXX no way to set tooltips here, kind of annoying
 
         # Hook up signals to get connection listing
-        self.engine.connect("connection-added", self.dest_add_connection)
-        self.engine.connect("connection-removed", self.dest_remove_connection)
+        self.engine.connect("conn-added", self.dest_add_conn)
+        self.engine.connect("conn-removed", self.dest_remove_conn)
         self.destconn_changed(dest_combo)
 
     def reset_state(self):
         title_str = ("<span size='large' color='white'>%s '%s'</span>" %
                      (_("Migrate"), self.vm.get_name()))
-        self.window.get_widget("migrate-main-label").set_markup(title_str)
+        self.widget("migrate-main-label").set_markup(title_str)
 
-        self.window.get_widget("migrate-cancel").grab_focus()
+        self.widget("migrate-cancel").grab_focus()
 
         name = self.vm.get_name()
         srchost = self.conn.get_hostname()
 
-        self.window.get_widget("migrate-label-name").set_text(name)
-        self.window.get_widget("migrate-label-src").set_text(srchost)
+        self.widget("migrate-label-name").set_text(name)
+        self.widget("migrate-label-src").set_text(srchost)
 
-        self.window.get_widget("migrate-advanced-expander").set_expanded(False)
-        self.window.get_widget("migrate-set-interface").set_active(False)
-        self.window.get_widget("migrate-set-rate").set_active(False)
-        self.window.get_widget("migrate-set-port").set_active(False)
-        self.window.get_widget("migrate-set-maxdowntime").set_active(False)
-        self.window.get_widget("migrate-max-downtime").set_value(30)
+        self.widget("migrate-set-interface").set_active(False)
+        self.widget("migrate-set-rate").set_active(False)
+        self.widget("migrate-set-port").set_active(False)
+        self.widget("migrate-set-maxdowntime").set_active(False)
+        self.widget("migrate-max-downtime").set_value(30)
 
         running = self.vm.is_active()
-        self.window.get_widget("migrate-offline").set_active(not running)
-        self.window.get_widget("migrate-offline").set_sensitive(running)
+        self.widget("migrate-offline").set_active(not running)
+        self.widget("migrate-offline").set_sensitive(running)
 
-        self.window.get_widget("migrate-rate").set_value(0)
-        self.window.get_widget("migrate-secure").set_active(False)
+        self.widget("migrate-rate").set_value(0)
+        self.widget("migrate-secure").set_active(False)
 
-        downtime_box = self.window.get_widget("migrate-maxdowntime-box")
+        downtime_box = self.widget("migrate-maxdowntime-box")
         support_downtime = self.vm.support_downtime()
         downtime_tooltip = ""
         if not support_downtime:
@@ -146,12 +157,12 @@ class vmmMigrateDialog(vmmGObjectUI):
 
         if self.conn.is_xen():
             # Default xen port is 8002
-            self.window.get_widget("migrate-port").set_value(8002)
+            self.widget("migrate-port").set_value(8002)
         else:
             # QEMU migrate port range is 49152+64
-            self.window.get_widget("migrate-port").set_value(49152)
+            self.widget("migrate-port").set_value(49152)
 
-        secure_box = self.window.get_widget("migrate-secure-box")
+        secure_box = self.widget("migrate-secure-box")
         support_secure = hasattr(libvirt, "VIR_MIGRATE_TUNNELLED")
         secure_tooltip = ""
         if not support_secure:
@@ -165,7 +176,7 @@ class vmmMigrateDialog(vmmGObjectUI):
 
     def set_state(self, vm):
         self.vm = vm
-        self.conn = vm.connection
+        self.conn = vm.conn
         self.reset_state()
 
     def destconn_changed(self, src):
@@ -174,31 +185,30 @@ class vmmMigrateDialog(vmmGObjectUI):
         if active == -1:
             tooltip = _("A valid destination connection must be selected.")
 
-        self.window.get_widget("migrate-finish").set_sensitive(active != -1)
-        util.tooltip_wrapper(self.window.get_widget("migrate-finish"), tooltip)
+        self.widget("migrate-finish").set_sensitive(active != -1)
+        util.tooltip_wrapper(self.widget("migrate-finish"), tooltip)
 
     def toggle_set_rate(self, src):
         enable = src.get_active()
-        self.window.get_widget("migrate-rate").set_sensitive(enable)
+        self.widget("migrate-rate").set_sensitive(enable)
 
     def toggle_set_interface(self, src):
         enable = src.get_active()
-        port_enable = self.window.get_widget("migrate-set-port").get_active()
-        self.window.get_widget("migrate-interface").set_sensitive(enable)
-        self.window.get_widget("migrate-set-port").set_sensitive(enable)
-        self.window.get_widget("migrate-port").set_sensitive(enable and
-                                                             port_enable)
+        port_enable = self.widget("migrate-set-port").get_active()
+        self.widget("migrate-interface").set_sensitive(enable)
+        self.widget("migrate-set-port").set_sensitive(enable)
+        self.widget("migrate-port").set_sensitive(enable and port_enable)
 
     def toggle_set_maxdowntime(self, src):
         enable = src.get_active()
-        self.window.get_widget("migrate-max-downtime").set_sensitive(enable)
+        self.widget("migrate-max-downtime").set_sensitive(enable)
 
     def toggle_set_port(self, src):
         enable = src.get_active()
-        self.window.get_widget("migrate-port").set_sensitive(enable)
+        self.widget("migrate-port").set_sensitive(enable)
 
     def get_config_destconn(self):
-        combo = self.window.get_widget("migrate-dest")
+        combo = self.widget("migrate-dest")
         model = combo.get_model()
 
         idx = combo.get_active()
@@ -212,39 +222,39 @@ class vmmMigrateDialog(vmmGObjectUI):
         return row[1]
 
     def get_config_offline(self):
-        return self.window.get_widget("migrate-offline").get_active()
+        return self.widget("migrate-offline").get_active()
 
     def get_config_max_downtime(self):
         if not self.get_config_max_downtime_enabled():
             return 0
-        return int(self.window.get_widget("migrate-max-downtime").get_value())
+        return int(self.widget("migrate-max-downtime").get_value())
 
     def get_config_secure(self):
-        return self.window.get_widget("migrate-secure").get_active()
+        return self.widget("migrate-secure").get_active()
 
     def get_config_max_downtime_enabled(self):
-        return self.window.get_widget("migrate-max-downtime").get_property("sensitive")
+        return self.widget("migrate-max-downtime").get_property("sensitive")
 
     def get_config_rate_enabled(self):
-        return self.window.get_widget("migrate-rate").get_property("sensitive")
+        return self.widget("migrate-rate").get_property("sensitive")
     def get_config_rate(self):
         if not self.get_config_rate_enabled():
             return 0
-        return int(self.window.get_widget("migrate-rate").get_value())
+        return int(self.widget("migrate-rate").get_value())
 
     def get_config_interface_enabled(self):
-        return self.window.get_widget("migrate-interface").get_property("sensitive")
+        return self.widget("migrate-interface").get_property("sensitive")
     def get_config_interface(self):
         if not self.get_config_interface_enabled():
             return None
-        return self.window.get_widget("migrate-interface").get_text()
+        return self.widget("migrate-interface").get_text()
 
     def get_config_port_enabled(self):
-        return self.window.get_widget("migrate-port").get_property("sensitive")
+        return self.widget("migrate-port").get_property("sensitive")
     def get_config_port(self):
         if not self.get_config_port_enabled():
             return 0
-        return int(self.window.get_widget("migrate-port").get_value())
+        return int(self.widget("migrate-port").get_value())
 
     def build_localhost_uri(self, destconn):
         desthost = destconn.get_qualified_hostname()
@@ -264,6 +274,18 @@ class vmmMigrateDialog(vmmGObjectUI):
         desturi_tuple[2] = desthost
         return uri_join(desturi_tuple)
 
+    def edit_uri(self, uri, hostname, port):
+        split = list(virtinst.util.uri_split(uri))
+
+        hostname = hostname or split[2]
+        if port:
+            if hostname.count(":"):
+                hostname = hostname.split(":")[0]
+            hostname += ":%s" % port
+
+        split[2] = hostname
+        return uri_join(tuple(split))
+
     def build_migrate_uri(self, destconn):
         conn = self.conn
 
@@ -271,27 +293,36 @@ class vmmMigrateDialog(vmmGObjectUI):
         port = self.get_config_port()
         secure = self.get_config_secure()
 
-        if not interface:
-            if not secure:
-                return None
+        if not interface and not secure:
+            return None
+
+        if secure:
+            # P2P migration uri is a libvirt connection uri, e.g.
+            # qemu+ssh://root@foobar/system
 
             # For secure migration, we need to make sure we aren't migrating
             # to the local connection, because libvirt will pull try to use
             # 'qemu:///system' as the migrate URI which will deadlock
             if destconn.is_local():
-                return self.build_localhost_uri(destconn)
+                uri = self.build_localhost_uri(destconn)
+            else:
+                uri = destconn.get_uri()
 
-        uri = ""
-        if conn.is_xen():
-            uri = "xenmigr://%s" % interface
+            uri = self.edit_uri(uri, interface, port)
 
         else:
-            uri = "tcp:%s" % interface
+            # Regular migration URI is HV specific
+            uri = ""
+            if conn.is_xen():
+                uri = "xenmigr://%s" % interface
 
-        if port:
-            uri += ":%s" % port
+            else:
+                uri = "tcp:%s" % interface
 
-        return uri
+            if port:
+                uri += ":%s" % port
+
+        return uri or None
 
     def rebuild_dest_rows(self):
         newrows = []
@@ -303,7 +334,7 @@ class vmmMigrateDialog(vmmGObjectUI):
         self.populate_dest_combo()
 
     def populate_dest_combo(self):
-        combo = self.window.get_widget("migrate-dest")
+        combo = self.widget("migrate-dest")
         model = combo.get_model()
         idx = combo.get_active()
         idxconn = None
@@ -338,8 +369,8 @@ class vmmMigrateDialog(vmmGObjectUI):
 
         combo.set_active(idx)
 
-    def dest_add_connection(self, engine_ignore, conn):
-        combo = self.window.get_widget("migrate-dest")
+    def dest_add_conn(self, engine_ignore, conn):
+        combo = self.widget("migrate-dest")
         model = combo.get_model()
 
         newrow = self.build_dest_row(conn)
@@ -353,10 +384,10 @@ class vmmMigrateDialog(vmmGObjectUI):
         self.destconn_rows.append(newrow)
         self.populate_dest_combo()
 
-    def dest_remove_connection(self, engine_ignore, conn):
+    def dest_remove_conn(self, engine_ignore, uri):
         # Make sure connection isn't already present
         for row in self.destconn_rows:
-            if row[1] and row[1].get_uri() == conn.get_uri():
+            if row[1] and row[1].get_uri() == uri:
                 self.destconn_rows.remove(row)
 
         self.populate_dest_combo()
@@ -419,7 +450,7 @@ class vmmMigrateDialog(vmmGObjectUI):
                 return
 
             destconn = self.get_config_destconn()
-            srchost = self.vm.get_connection().get_hostname()
+            srchost = self.vm.conn.get_hostname()
             dsthost = destconn.get_qualified_hostname()
             max_downtime = self.get_config_max_downtime()
             live = not self.get_config_offline()
@@ -431,7 +462,8 @@ class vmmMigrateDialog(vmmGObjectUI):
         except Exception, e:
             details = "".join(traceback.format_exc())
             self.err.show_err((_("Uncaught error validating input: %s") %
-                               str(e)), details)
+                               str(e)),
+                               details=details)
             return
 
         self.topwin.set_sensitive(False)
@@ -451,6 +483,7 @@ class vmmMigrateDialog(vmmGObjectUI):
                               (_("Migrating VM '%s' from %s to %s. "
                                  "This may take awhile.") %
                                 (self.vm.get_name(), srchost, dsthost)),
+                              self.topwin,
                               cancel_back=_cancel_back,
                               cancel_args=_cancel_args)
         error, details = progWin.run()
@@ -460,7 +493,8 @@ class vmmMigrateDialog(vmmGObjectUI):
 
         if error:
             error = _("Unable to migrate guest: %s") % error
-            self.err.show_err(error, error + "\n" + details)
+            self.err.show_err(error,
+                              details=details)
         else:
             self.conn.tick(noStatsUpdate=True)
             destconn.tick(noStatsUpdate=True)
@@ -501,7 +535,7 @@ class vmmMigrateDialog(vmmGObjectUI):
                        secure, max_downtime):
         meter = vmmCreateMeter(asyncjob)
 
-        srcconn = util.dup_conn(origvm.get_connection())
+        srcconn = util.dup_conn(origvm.conn)
         dstconn = util.dup_conn(origdconn)
 
         vminst = srcconn.vmm.lookupByName(origvm.get_name())
@@ -515,7 +549,7 @@ class vmmMigrateDialog(vmmGObjectUI):
             # 0 means that the spin box migrate-max-downtime does not
             # be enabled.
             current_thread = threading.currentThread()
-            timer = util.safe_timeout_add(100,
+            timer = self.safe_timeout_add(100,
                                           self._async_set_max_downtime,
                                           vm, max_downtime,
                                           current_thread)
