@@ -18,7 +18,6 @@
 # MA 02110-1301 USA.
 #
 
-import gobject
 import gtk
 
 import virtManager.util as util
@@ -28,35 +27,30 @@ PREFS_PAGE_STATS    = 0
 PREFS_PAGE_VM_PREFS = 1
 
 class vmmPreferences(vmmGObjectUI):
-    __gsignals__ = {
-        "action-show-help": (gobject.SIGNAL_RUN_FIRST,
-                             gobject.TYPE_NONE, [str]),
-        }
     def __init__(self):
         vmmGObjectUI.__init__(self, "vmm-preferences.glade", "vmm-preferences")
 
-        self.config.on_view_system_tray_changed(self.refresh_view_system_tray)
-        self.config.on_console_popup_changed(self.refresh_console_popup)
-        self.config.on_console_accels_changed(self.refresh_console_accels)
-        self.config.on_console_scaling_changed(self.refresh_console_scaling)
-        self.config.on_stats_update_interval_changed(self.refresh_update_interval)
-        self.config.on_stats_history_length_changed(self.refresh_history_length)
-        self.config.on_sound_local_changed(self.refresh_sound_local)
-        self.config.on_sound_remote_changed(self.refresh_sound_remote)
-        self.config.on_graphics_type_changed(self.refresh_graphics_type)
-        self.config.on_stats_enable_disk_poll_changed(self.refresh_disk_poll)
-        self.config.on_stats_enable_net_poll_changed(self.refresh_net_poll)
+        self.add_gconf_handle(self.config.on_view_system_tray_changed(self.refresh_view_system_tray))
+        self.add_gconf_handle(self.config.on_console_accels_changed(self.refresh_console_accels))
+        self.add_gconf_handle(self.config.on_console_scaling_changed(self.refresh_console_scaling))
+        self.add_gconf_handle(self.config.on_stats_update_interval_changed(self.refresh_update_interval))
+        self.add_gconf_handle(self.config.on_stats_history_length_changed(self.refresh_history_length))
+        self.add_gconf_handle(self.config.on_sound_local_changed(self.refresh_sound_local))
+        self.add_gconf_handle(self.config.on_sound_remote_changed(self.refresh_sound_remote))
+        self.add_gconf_handle(self.config.on_graphics_type_changed(self.refresh_graphics_type))
+        self.add_gconf_handle(self.config.on_stats_enable_disk_poll_changed(self.refresh_disk_poll))
+        self.add_gconf_handle(self.config.on_stats_enable_net_poll_changed(self.refresh_net_poll))
 
-        self.config.on_confirm_forcepoweroff_changed(self.refresh_confirm_forcepoweroff)
-        self.config.on_confirm_poweroff_changed(self.refresh_confirm_poweroff)
-        self.config.on_confirm_pause_changed(self.refresh_confirm_pause)
-        self.config.on_confirm_removedev_changed(self.refresh_confirm_removedev)
-        self.config.on_confirm_interface_changed(self.refresh_confirm_interface)
+        self.add_gconf_handle(self.config.on_confirm_forcepoweroff_changed(self.refresh_confirm_forcepoweroff))
+        self.add_gconf_handle(self.config.on_confirm_poweroff_changed(self.refresh_confirm_poweroff))
+        self.add_gconf_handle(self.config.on_confirm_pause_changed(self.refresh_confirm_pause))
+        self.add_gconf_handle(self.config.on_confirm_removedev_changed(self.refresh_confirm_removedev))
+        self.add_gconf_handle(self.config.on_confirm_interface_changed(self.refresh_confirm_interface))
+        self.add_gconf_handle(self.config.on_confirm_unapplied_changed(self.refresh_confirm_unapplied))
 
         self.refresh_view_system_tray()
         self.refresh_update_interval()
         self.refresh_history_length()
-        self.refresh_console_popup()
         self.refresh_console_accels()
         self.refresh_console_scaling()
         self.refresh_sound_local()
@@ -70,12 +64,12 @@ class vmmPreferences(vmmGObjectUI):
         self.refresh_confirm_pause()
         self.refresh_confirm_removedev()
         self.refresh_confirm_interface()
+        self.refresh_confirm_unapplied()
 
         self.window.signal_autoconnect({
             "on_prefs_system_tray_toggled" : self.change_view_system_tray,
             "on_prefs_stats_update_interval_changed": self.change_update_interval,
             "on_prefs_stats_history_length_changed": self.change_history_length,
-            "on_prefs_console_popup_changed": self.change_console_popup,
             "on_prefs_console_accels_toggled": self.change_console_accels,
             "on_prefs_console_scaling_changed": self.change_console_scaling,
             "on_prefs_close_clicked": self.close,
@@ -90,21 +84,26 @@ class vmmPreferences(vmmGObjectUI):
             "on_prefs_confirm_pause_toggled": self.change_confirm_pause,
             "on_prefs_confirm_removedev_toggled": self.change_confirm_removedev,
             "on_prefs_confirm_interface_toggled": self.change_confirm_interface,
+            "on_prefs_confirm_unapplied_toggled": self.change_confirm_unapplied,
             "on_prefs_btn_keys_define_clicked": self.change_grab_keys,
             "on_prefs_graphics_type_changed": self.change_graphics_type,
             })
-        util.bind_escape_key_close(self)
+        self.bind_escape_key_close()
 
         # XXX: Help docs useless/out of date
-        self.window.get_widget("prefs-help").hide()
+        self.widget("prefs-help").hide()
 
     def close(self, ignore1=None, ignore2=None):
         self.topwin.hide()
         return 1
 
-    def show(self):
+    def show(self, parent):
+        self.topwin.set_transient_for(parent)
         self.topwin.present()
 
+    def _cleanup(self):
+        pass
+
     #########################
     # Config Change Options #
     #########################
@@ -112,43 +111,39 @@ class vmmPreferences(vmmGObjectUI):
     def refresh_view_system_tray(self, ignore1=None, ignore2=None,
                                  ignore3=None, ignore4=None):
         val = self.config.get_view_system_tray()
-        self.window.get_widget("prefs-system-tray").set_active(bool(val))
+        self.widget("prefs-system-tray").set_active(bool(val))
 
     def refresh_update_interval(self, ignore1=None, ignore2=None,
                                 ignore3=None, ignore4=None):
-        self.window.get_widget("prefs-stats-update-interval").set_value(
+        self.widget("prefs-stats-update-interval").set_value(
             self.config.get_stats_update_interval())
     def refresh_history_length(self, ignore1=None, ignore2=None,
                                ignore3=None, ignore4=None):
-        self.window.get_widget("prefs-stats-history-len").set_value(
+        self.widget("prefs-stats-history-len").set_value(
             self.config.get_stats_history_length())
 
-    def refresh_console_popup(self, ignore1=None, ignore2=None,
-                              ignore3=None, ignore4=None):
-        self.window.get_widget("prefs-console-popup").set_active(
-            self.config.get_console_popup())
     def refresh_console_accels(self, ignore1=None, ignore2=None,
                                 ignore3=None, ignore4=None):
-        self.window.get_widget("prefs-console-accels").set_active(
+        self.widget("prefs-console-accels").set_active(
             self.config.get_console_accels())
     def refresh_console_scaling(self, ignore1=None, ignore2=None,
                                 ignore3=None, ignore4=None):
         val = self.config.get_console_scaling()
         if val == None:
             val = 0
-        self.window.get_widget("prefs-console-scaling").set_active(val)
+        self.widget("prefs-console-scaling").set_active(val)
 
     def refresh_sound_local(self, ignore1=None, ignore2=None, ignore=None,
                             ignore4=None):
-        self.window.get_widget("prefs-sound-local").set_active(
+        self.widget("prefs-sound-local").set_active(
             self.config.get_local_sound())
     def refresh_sound_remote(self, ignore1=None, ignore2=None, ignore=None,
                              ignore4=None):
-        self.window.get_widget("prefs-sound-remote").set_active(
+        self.widget("prefs-sound-remote").set_active(
             self.config.get_remote_sound())
     def refresh_graphics_type(self, ignore1=None, ignore2=None, ignore=None,
                              ignore4=None):
-        combo = self.window.get_widget("prefs-graphics-type")
+        combo = self.widget("prefs-graphics-type")
         model = combo.get_model()
         gtype = self.config.get_graphics_type()
 
@@ -159,15 +154,15 @@ class vmmPreferences(vmmGObjectUI):
                 active = rowidx
                 break
 
-        self.window.get_widget("prefs-graphics-type").set_active(active)
+        self.widget("prefs-graphics-type").set_active(active)
 
     def refresh_disk_poll(self, ignore1=None, ignore2=None, ignore3=None,
                           ignore4=None):
-        self.window.get_widget("prefs-stats-enable-disk").set_active(
+        self.widget("prefs-stats-enable-disk").set_active(
             self.config.get_stats_enable_disk_poll())
     def refresh_net_poll(self, ignore1=None, ignore2=None, ignore3=None,
                          ignore4=None):
-        self.window.get_widget("prefs-stats-enable-net").set_active(
+        self.widget("prefs-stats-enable-net").set_active(
             self.config.get_stats_enable_net_poll())
 
     def refresh_grabkeys_combination(self, ignore1=None, ignore2=None,
@@ -176,8 +171,8 @@ class vmmPreferences(vmmGObjectUI):
         if val is None:
             val = "Control_L+Alt_L"
 
-        prefs_button = self.window.get_widget("prefs-keys-grab-changebtn")
-        self.window.get_widget("prefs-keys-grab-sequence").set_text(val)
+        prefs_button = self.widget("prefs-keys-grab-changebtn")
+        self.widget("prefs-keys-grab-sequence").set_text(val)
         if not self.config.vnc_grab_keys_supported():
             util.tooltip_wrapper(prefs_button,
                                  _("Installed version of GTK-VNC doesn't "
@@ -185,19 +180,28 @@ class vmmPreferences(vmmGObjectUI):
 
     def refresh_confirm_forcepoweroff(self, ignore1=None, ignore2=None,
                                       ignore3=None, ignore4=None):
-        self.window.get_widget("prefs-confirm-forcepoweroff").set_active(self.config.get_confirm_forcepoweroff())
+        self.widget("prefs-confirm-forcepoweroff").set_active(
+                                self.config.get_confirm_forcepoweroff())
     def refresh_confirm_poweroff(self, ignore1=None, ignore2=None,
                                       ignore3=None, ignore4=None):
-        self.window.get_widget("prefs-confirm-poweroff").set_active(self.config.get_confirm_poweroff())
+        self.widget("prefs-confirm-poweroff").set_active(
+                                self.config.get_confirm_poweroff())
     def refresh_confirm_pause(self, ignore1=None, ignore2=None,
                               ignore3=None, ignore4=None):
-        self.window.get_widget("prefs-confirm-pause").set_active(self.config.get_confirm_pause())
+        self.widget("prefs-confirm-pause").set_active(
+                                self.config.get_confirm_pause())
     def refresh_confirm_removedev(self, ignore1=None, ignore2=None,
                                   ignore3=None, ignore4=None):
-        self.window.get_widget("prefs-confirm-removedev").set_active(self.config.get_confirm_removedev())
+        self.widget("prefs-confirm-removedev").set_active(
+                                self.config.get_confirm_removedev())
     def refresh_confirm_interface(self, ignore1=None, ignore2=None,
                                   ignore3=None, ignore4=None):
-        self.window.get_widget("prefs-confirm-interface").set_active(self.config.get_confirm_interface())
+        self.widget("prefs-confirm-interface").set_active(
+                                self.config.get_confirm_interface())
+    def refresh_confirm_unapplied(self, ignore1=None, ignore2=None,
+                                  ignore3=None, ignore4=None):
+        self.widget("prefs-confirm-unapplied").set_active(
+                                self.config.get_confirm_unapplied())
 
     def grabkeys_get_string(self, keysyms):
         keystr = None
@@ -228,19 +232,26 @@ class vmmPreferences(vmmGObjectUI):
 
     def change_grab_keys(self, src_ignore):
         dialog = gtk.Dialog(_("Configure grab key combination"),
-                            None,
+                            self.topwin,
                             gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                             (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                              gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
-        label = gtk.Label(_("Please press desired grab key combination"))
-        dialog.set_size_request(325, 160)
-        (dialog.get_content_area()).add(
-            gtk.Label(_("You can now define grab keys by pressing them.\n"
-                        "To confirm your selection please click OK button\n"
-                        "while you have desired keys pressed.")))
+        dialog.set_default_size(325, 160)
+        dialog.set_border_width(6)
 
-        (dialog.get_content_area()).add(label)
-        defs = {'label': label, 'keysyms': []}
+        infolabel = gtk.Label(
+                    _("You can now define grab keys by pressing them.\n"
+                      "To confirm your selection please click OK button\n"
+                      "while you have desired keys pressed."))
+        keylabel = gtk.Label(_("Please press desired grab key combination"))
+
+        vbox = gtk.VBox()
+        vbox.set_spacing(12)
+        vbox.pack_start(infolabel, False, False)
+        vbox.pack_start(keylabel, False, False)
+        dialog.get_content_area().add(vbox)
+
+        defs = {'label': keylabel, 'keysyms': []}
         dialog.connect("key-press-event", self.grabkeys_dlg_press, defs)
         dialog.connect("key-release-event", self.grabkeys_dlg_release, defs)
         dialog.show_all()
@@ -260,8 +271,6 @@ class vmmPreferences(vmmGObjectUI):
     def change_history_length(self, src):
         self.config.set_stats_history_length(src.get_value_as_int())
 
-    def change_console_popup(self, box):
-        self.config.set_console_popup(box.get_active())
     def change_console_accels(self, src):
         self.config.set_console_accels(src.get_active())
     def change_console_scaling(self, box):
@@ -287,6 +296,8 @@ class vmmPreferences(vmmGObjectUI):
         self.config.set_confirm_removedev(src.get_active())
     def change_confirm_interface(self, src):
         self.config.set_confirm_interface(src.get_active())
+    def change_confirm_unapplied(self, src):
+        self.config.set_confirm_unapplied(src.get_active())
 
     def change_graphics_type(self, src):
         gtype = 'vnc'
@@ -299,3 +310,6 @@ class vmmPreferences(vmmGObjectUI):
         # From the Preferences window, show the help document from
         # the Preferences page
         self.emit("action-show-help", "virt-manager-preferences-window")
+
+vmmPreferences.type_register(vmmPreferences)
+vmmPreferences.signal_new(vmmPreferences, "action-show-help", [str])
